package org.bitbucket.bradleysmithllc.etlunit.feature.informatica;

import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.*;
import org.bitbucket.bradleysmithllc.etlunit.Log;
import org.bitbucket.bradleysmithllc.etlunit.NullClassDirector;
import org.bitbucket.bradleysmithllc.etlunit.context.VariableContext;
import org.bitbucket.bradleysmithllc.etlunit.feature.RuntimeOption;
import org.bitbucket.bradleysmithllc.etlunit.metadata.MetaDataArtifact;
import org.bitbucket.bradleysmithllc.etlunit.metadata.MetaDataContext;
import org.bitbucket.bradleysmithllc.etlunit.metadata.MetaDataPackageContext;
import org.bitbucket.bradleysmithllc.etlunit.parser.ETLTestValueObject;

import javax.inject.Inject;
import javax.inject.Named;
import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.Map;

public class InformaticaDirector extends NullClassDirector {
	private Log applicationLog;

	private boolean loadWorkflows = true;
	private boolean cleanRepository = false;

	private VariableContext variableContext;
	private InformaticaConfiguration informaticaConfiguration;
	private InformaticaFeatureModule informaticaFeatureModule;

	private InformaticaRuntimeSupport informaticaRuntimeSupport;

	@Inject
	public void setInformaticaRuntimeSupport(InformaticaRuntimeSupport support) {
		informaticaRuntimeSupport = support;
	}

	@Inject
	public void receiveInformaticaFeatureModule(InformaticaFeatureModule icon) {
		informaticaFeatureModule = icon;
	}

	@Inject
	public void receiveInformaticaConfiguration(InformaticaConfiguration icon) {
		informaticaConfiguration = icon;
	}

	@Inject
	public void setLogger(VariableContext vcon) {
		variableContext = vcon;
	}

	@Inject
	public void setLogger(@Named("applicationLog") Log logger) {
		this.applicationLog = logger;
	}

	@Inject
	public void setCleanRepository(@Named("informatica.cleanRepository") RuntimeOption option) {
		cleanRepository = option.isEnabled();

		applicationLog.info(cleanRepository ? "Cleaning repository." : "Leaving turds in repository.");
	}

	@Inject
	public void setLoadWorkflows(@Named("informatica.loadWorkflows") RuntimeOption option) {
		loadWorkflows = option.isEnabled();

		applicationLog.info(loadWorkflows ? "Loading workflows from project" : "Using workflows resident in informatica repository");
	}

	@Override
	public void endBroadcast() {
		InformaticaRepositoryClientImpl.resetClientCount();

		if (cleanRepository) {
			ETLTestValueObject container = InformaticaExecutor.getInformaticaContainer(variableContext, false);

			if (container != null) {
				Map<String, ETLTestValueObject> map = container.getValueAsMap();

				for (Map.Entry<String, ETLTestValueObject> entry : map.entrySet()) {
					applicationLog.info("Purging temporary informatica folders for domain: " + entry.getKey());

					InformaticaDomain domain = informaticaConfiguration.getDomain(entry.getKey());

					Map<String, ETLTestValueObject> mapRepo = entry.getValue().getValueAsMap();

					for (Map.Entry<String, ETLTestValueObject> entryRepo : mapRepo.entrySet()) {
						applicationLog.info("Purging temporary informatica folders for repository: " + entryRepo.getKey());

						InformaticaRepository repository = domain.getRepository(entryRepo.getKey());

						InformaticaRepositoryClient client = null;
						try {
							client = repository.getInformaticaRepositoryClient();

							if (loadWorkflows) {
								applicationLog.info("Purging temporary informatica folders.");

								Map<String, ETLTestValueObject> fMap = entryRepo.getValue().query(InformaticaExecutor.INFORMATICA_CREATED_FOLDERS).getValueAsMap();

								for (Map.Entry<String, ETLTestValueObject> folderEntry : fMap.entrySet()) {
									applicationLog.info("Purging temporary informatica folder: " + folderEntry.getKey());

									client.deleteFolder(folderEntry.getKey());
								}
							} else {
								applicationLog.info("Nothing to clean up for repository " + repository.getQualifiedName() + " since load workflows is turned off.");
							}

							Map<String, ETLTestValueObject> fMap = entryRepo.getValue().query(InformaticaExecutor.INFORMATICA_CREATED_CONNECTIONS).getValueAsMap();

							for (Map.Entry<String, ETLTestValueObject> connectionEntry : fMap.entrySet()) {
								applicationLog.info("Purging temporary informatica folder: " + connectionEntry.getKey());

								client.deleteFolder(connectionEntry.getKey());
							}
						} catch (Exception testExecutionError) {
							throw new RuntimeException(testExecutionError);
						}
					}
				}
			}

			applicationLog.info("Purging connection objects.");

		} else {
			applicationLog.info("Not cleaning repository.");
		}
	}

	@Override
	public void beginBroadcast() {
		MetaDataContext mc = informaticaFeatureModule.getInformaticaMetaContext();
		MetaDataPackageContext pc = mc.createPackageContext(null, MetaDataPackageContext.path_type.feature_source);

		// create a package, artifact and contents for the informatica workflows and parameter files
		File infaRoot = informaticaRuntimeSupport.getInformaticaSourceDirectory();

		try {
			for (File infaFolder : infaRoot.listFiles(new FileFilter() {
				@Override
				public boolean accept(File pathname) {
					// we don't want .svn directories
					return pathname.isDirectory() && !pathname.getName().startsWith(("."));
				}
			})) {
				// each entry here is an informatica folder with workflows and a ParameterFiles directory
				final MetaDataArtifact art = pc.createArtifact(infaFolder.getName(), infaFolder);

				// grab the contents
				infaFolder.listFiles(new FileFilter() {
					@Override
					public boolean accept(File pathname) {
						if (pathname.getName().toLowerCase().endsWith(".xml"))
						{
							art.createContent(pathname.getName());
						}

						return false;
					}
				});

				File informaticaWorkflowParameterDirectory = informaticaRuntimeSupport.getInformaticaWorkflowParameterDirectory(infaFolder.getName());
				final MetaDataArtifact artParm = pc.createArtifact(infaFolder.getName() + "/ParameterFiles", informaticaWorkflowParameterDirectory);

				// grab the contents
				informaticaWorkflowParameterDirectory.listFiles(new FileFilter() {
					@Override
					public boolean accept(File pathname) {
						if (pathname.getName().toLowerCase().endsWith(".prm")) {
							artParm.createContent(pathname.getName());
						}

						return false;
					}
				});
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
}